function [ToolboxesPresent] = checkToolboxes( toolboxes )
%CHECKTOOLBOXES - Checks whether certain toolboxes are installed
%
%   Syntax:
%       output = CHECKTOOLBOXES({toolbox1,toolbox2})
%
%   Description:
%       CHECKTOOLBOXES( toolboxes ) - Verifies whether each of the
%       toolboxes specified by the cell array 'toolboxes' is present in the
%       current MATLAB installation by searching trough the output of the
%       MATLAB function 'ver'. If all specified toolboxes are installed,
%       nothing happens. If one or more toolbox is missing, a message is
%       shown, indicating the name of the missing toolbox.
%    
%   Inputs:
%       toolboxes - Cell array containing scripts, each of which details
%                   the exact name of a toolbox.
%
%   Outputs:
%       ToolboxPresent - boolean indicating all toolboxes are present. 
%           ToolboxPresent = true: all toolboxes are present
%           ToolboxPresent = false: one or more toolboxes are missing.
%
%   Other m-files required: none
%   Subfunctions: none
%   MAT-files required: none
%
%   See also: ver

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 06-Oct-2020; Last revision: 06-Nov-2020
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved.

VersionInfo = ver;
ToolboxesPresent = true;
MissingToolbox = '';
for i = 1:numel(toolboxes)
    if any(strcmp({VersionInfo.Name},toolboxes{i})) == false
        ToolboxesPresent = false;
        MissingToolbox = [MissingToolbox toolboxes{i}];
    end
end
if ToolboxesPresent == false
    disp(['The ' MissingToolbox ' is not installed on the current system. This toolbox is not ' newline...
          'required, but it is recommended. Alternative functions will be used.']);
end