function Outputfile = downloadOSMmap( route, OSMfolder )
%DOWNLOADOSMMAP - Downloads Openstreetmap information via the
%Overpass-turbo API.
%
%   Syntax:
%       Outputfile = DOWNLOADOSMMAP(route, OSMfolder)
%
%   Description:
%       DOWNLOADOSMMAP(route, OSMfolder) - Downloads OpenStreetMap
%       information, in the form of an .osm-file, via the Overpass-turbo
%       API.
%       Before downloading, the function checks whether a local map might
%       already be available in the folder specified by 'OSMfolder'. 
%       Only map information within a bounding box eveloping the route will
%       be downloaded. Additionally, only map information related to roads
%       (intended for motorized vehicles) will be downloaded. Both these
%       measures are to reduce the filesize of the downloaded .osm-file.
%    
%   Inputs:
%       route - A struct containing at least the following fields:
%           route.lat - Array of doubles indicating latitudes [deg]
%           route.lon - Array of doubles indicatings longitudes [deg]
%       Together, route.lat and route.lon describe the coordinates of a
%       route.
%       OSMfolder - String specifiying the location where the .osm-file
%       is to be stored.
%
%   Outputs:
%       Outputfile - String specifying the path to the .osm-file. The
%       output file wil baer a unique filename, which is based on the
%       coordinates of the requested route.
%
%   Other m-files required: none
%   Subfunctions: none
%   MAT-files required: none
%
%   See also: webread
%
% Source: https://blog-en.openalfa.com/how-to-query-openstreetmap-using-the-overpass-api
%         https://wiki.openstreetmap.org/wiki/Overpass_API/Language_Guide

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 21-Aug-2019; Last revision: 22-Dec-2020
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved

%% Load data
lat = route.lat;
lon = route.lon;

%Define maximum lat,lon-range:
latlonrange = ['' num2str(min(lat)) ',' ...
                  num2str(min(lon)) ',' ...
                  num2str(max(lat)) ',' ...
                  num2str(max(lon))''];
              
%Construct unique filenames based on an ID number that contains coordinates
IDnr = latlonrange;
IDnr(regexp(IDnr,'[,,.]'))=[];
Outputfile = fullfile(OSMfolder,['OSMmap_' IDnr '.osm']);

%define API url
APIurl = 'http://overpass-api.de/api/interpreter?';
webOptions = weboptions('Timeout',25);

%% If map already exists, as user to load it or not:
loadbool = [];  %boolean indicating whether a new map should be downloaded
if exist(Outputfile,'file')

    %Extract file date and time:
    FileInfo = dir(Outputfile);
    TimeStamp = FileInfo.date;
    
    %Ask user to replace route file or not
    disp(['Map was found on local disk: ' Outputfile '']);
    prompt = ['This map file was created on ' TimeStamp '.    Do you want to update it? Y/N: '];
    while isempty(loadbool) %while no answer has been defined:
        
        str = input(prompt,'s'); %Prompt user
        
        if strcmpi(str,'Y') %If user says 'yes':
            loadbool = true;
        elseif strcmpi(str,'N') %If user says 'no':
            loadbool = false;
        end
    end
    
else %No map file is present, in that case: download a new one in any case:
    loadbool = true;
end

if loadbool == true     

    %download OSM data:
    disp('Downloading map data from OpenStreetMap...');
     
    %Check whether Overpass turbo is online:
    try
        java.net.InetAddress.getByName('overpass-api.de');
    catch
        error('It appears that the overpass-turbo API can currently not be reached');
    end
    
    %Specify query details (only load particu
    query = ['('...
        'way["highway"~"."]'...
        '["highway"!="footway"]'...
        '["highway"!="pedestrian"]'...
        '["highway"!="path"]'...
        '["highway"!="cycleway"]'...
        '('  latlonrange ');>;'...
        ');'...
        'out;'];
    try
        XMLmap = webread(APIurl,'data',query,webOptions); %download map via Overpass API
    catch ME
        error(['It seems the Overpass-Turbo API could not resolve the requested map data.' newline...
            'Trying to call the API resulted in the following error: ' newline newline ME.message ]);
    end
    
    %append bounds statement at the beginning of the xml file:
    BoundsStatement = ['<bounds minlat="' num2str(min(lat)) '" minlon="' num2str(min(lon)) '" maxlat="' num2str(max(lat)) '" maxlon="' num2str(max(lon)) '"/>'];
    Index = strfind(XMLmap,'<meta osm_base=');
    
    %Include bounds statement:
    XMLmap = [XMLmap(1:Index+38)...
        newline...
        BoundsStatement...
        XMLmap(Index+40:end)];
    
    %Write xml data to file:
    fileID = fopen(Outputfile,'w');
    fprintf(fileID,'%s',XMLmap);
    fclose(fileID);
    
    disp(['Map succesfully downloaded and saved as: ' Outputfile '.']);
    
    %Also delete any existing parsed map file (a new one will be
    %generated shortly)
    ParsedOutputFile = [Outputfile(1:end-4) '.mat'];
    if exist(ParsedOutputFile,'file')
        delete(ParsedOutputFile);
    end
    
end

end