function sol = energyConsumptionPrediction( trk, veh, env, set )
%ENERGYCONSUMPTIONPREDICTION - Performs a physics-based energy consumption
%prediction of an electric vehicle by estimating road-load forces, such as
%aerodynamic drag and rolling resistance.
%
%   Syntax:
%       sol = ENERGYCONSUMPTIONPREDICTION( trk, veh, env, settings )
%
%   Description:
%       ENERGYCONSUMPTIONPREDICTION() - Performs a physics-based energy
%       consumption prediction of the vehicle defined by 'veh', driving
%       along a track defined by 'trk', with environmental parameters as
%       defined in 'env'. The prediction relies on a physical description
%       of the vehicle
%       experiencing various resistance forces:
%           - Rolling Resistance
%           - A longitudinal component of gravity
%           - Aerodynamic Resistance
%           - An acceleration related component
%       The script is structured as follows: 
%           1 Renaming of some variables
%           2 Pre-processing. If any parameters are to be variable,
%             according to the settings provided by the user, these are
%             caluclated here
%           3 The acutal energy onsumption prediction
%           4 Saving the results to a solution-sturct named 'sol'
%       
%   Inputs:
%       trk - struct describing the 
%           trk.time: [s] trip time
%           trk.dis : [m] travelled distance
%           trk.vel : [m/s] vehicle velocity
%           trk.acc : [m/s^2] vehicle acceleration 
%           trk.grad: [rad] road gradient
%       veh - struct with vehicle parameters according to
%           parameters_vehicle.m
%       env - struct environmental parameters according to 
%           parameters_environment.m
%       set - struct with the following booleans to indicate whether
%           certain model parameters should be considered variable:
%           set.includeSlope
%           set.includeWeather 
%           set.variableFr
%
%   Outputs:
%       sol - struct describing the solution of the energy consumption
%           prediction. In detail, sol contains the fields:
%           sol.time     : [s] trip time
%           sol.dis      : [m] travelled distance
%           sol.vel      : [m/s] vehicle velocity
%           sol.acc      : [m/s^2] vehicle acceleration 
%           sol.w_whl    : [rad/s] angular velocity wheels
%           sol.T_whl    : [Nm] wheel torque delivered by powertrain
%           sol.Pptloss  : [W] powertrain losses
%           sol.HydrBrake: [-] hydraulic braking boolean (1 = hydraulic
%                          braking)
%           sol.Pinv     : [W] Inverter DC power
%           sol.Pbat     : [W] Battery power (positive is dischrage)
%           sol.E_discharge: [J] Battery discharge energy
%           sol.E_regen  : [J] Battery regen energy
%
%   Other m-files required: getVariableFr, Ppt_loss_eVan
%   Subfunctions: none
%   MAT-files required: none
%
%   See also: getVariableFr, Ppt_loss_eVan, parameters_vehicle, 
%   parameters_environment 

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 18-Jul-2019; Last revision: 01-Sep-2020
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved.

%% 1. Rename variables
disp('================== Energy Consumption Prediction has been started.. ==================');

%rename inputs
t = trk.time;     % [s] time
dis = trk.dis;    % [m] distance
vel = trk.vel;    % [m/s] vehicle velocity
acc = trk.acc;    % [m/s^2] vehicle acceleration
n = numel(vel);   % number of data points

%Correct for the offset between velocity (which is known 
t_avg =   t(1:n-1)+0.5*diff( t ); % [s] average time point between to data points
d_avg = dis(1:n-1)+0.5*diff(dis); % [m] distance
v_avg = vel(1:n-1)+0.5*diff(vel); % [m/s] average velocity bewtween 2 data points
dt    = diff(t);                  % [s] sample time

% Determine rotational velocities
w_whl     = v_avg ./ veh.rTyre;     %Angular velocity wheel [rad/s]
w_mot     = w_whl .* veh.gearRatio; %Angular velocity EM [rad/s]

%% 2. PRE-PROCESSING: Determine possible non-constant variables

% slope
if set.includeSlope
    if ~(isfield(trk,'grad'))
        warning('Road gradient data not found');
    end
    grad = trk.grad;
    grad(isnan(grad)) = 0;
   
    disp('Variable road gradient (based on SRTM) is used:      yes');
    
else % assume gradient zero
    grad = zeros(n-1,1);
    
    disp('Variable road gradient (based on SRTM) is used:      no (slope assumed zero)');
end

% rolling resistance
if set.variableFr
    
    fr = getVariableFr(v_avg,veh);
    disp('Variable rolling resistance is used:                 yes');
    
else % assume rolling resistance constant
    fr = veh.fr_mid;
    
    disp('Variable rolling resistance is used:                 no (only value for fr_mid is used)');
    
end

% wind angle
if set.includeWeather && (isfield(trk,'relWindMag') && isfield(trk,'relWindBeta'))
    
    NanIndices = isnan(trk.relWindMag) | isnan(trk.relWindBeta);
    
    beta = trk.relWindBeta';
    beta(NanIndices) = 0; %If no info: assume headwind
    
    v_wind = trk.relWindMag'.*cosd(beta);
    v_wind(NanIndices) = v_avg(NanIndices); %If no info: assume wind speed is equal to vehicle speed.
    
    disp('Wind influence and variable airdensity are included: yes');
    
else
    beta = 0;
    v_wind = v_avg;
    disp('Wind influence and variable airdensity are included: no (no wind and default air density)');
end

%% 3. MAIN CALCULATIONS: Determine road-load forces
F_rr    = fr .* (veh.mVehicle + veh.mCargo) .* env.g  .* cos(grad) .* tanh(100.*v_avg); %Rolling resistance [N]
F_slope = sin(grad) .* (veh.mVehicle + veh.mCargo) .* env.g;                            %Force due to slope [N]
F_aero  = 1/2 * env.rho * (veh.Cd + veh.DeltaCd(beta)) .* veh.Af .* v_wind.^2;          %Aerodynamic drag force [N]
F_acc   = (veh.mEff + veh.mCargo) * acc;                                                %Inertial force vehicle [N]                    

%Determine total driving force at wheel (F_drive > 0 = forward force acting
%on vehicle)
F_drive = F_rr + F_slope + F_aero + F_acc;                       %Road load force at the wheel [N]

% Determine Power and Torque supplied by driveline:
HydrBrakBool = F_drive < veh.Fwhl_regenLimit(v_avg);            %[-] Boolean indicating whether hydraulic brake has been applied
T_whl   = max(F_drive,veh.Fwhl_regenLimit(v_avg)) .* veh.rTyre; %[Nm] Wheel torque delivered by powertrain
P_whl   = T_whl.*w_whl;                                         %[W] effective delivered driveline power to wheel

%Determine driveline losses:
Pptloss = Ppt_loss_eVan( w_whl, T_whl );
Pptloss(isnan(Pptloss)) = 0; %catch nan values

%Determine total inverter and battery losses:
Pinv = P_whl + Pptloss;   %Driveline Inverter power (on DC side) [W]
Pbat = Pinv  + veh.P_aux; %Battery power [W]

%% 4. POST-PROCESSING: Store the solution
sol.time        = t_avg; %[s] time
sol.dis         = d_avg; %[m] distance
sol.vel         = v_avg; %[m/s] velocity
sol.acc         = acc;   %[m/s^2] acceleration 
sol.w_whl       = w_whl; %[rad/s] Angular velocity wheel
sol.T_whl       = T_whl; %[Nm] Wheel torque delivered by powertrain
sol.Pptloss     = Pptloss;%[W] Powertrain losses
sol.HydrBrake   = HydrBrakBool;%[-] Hydraulic braking boolean
sol.Pinv        = Pinv; %[W] Inverter DC power
sol.Pbat        = Pbat; %[W] Battery power

P_discharge = Pbat;
P_discharge(Pbat<0) = 0;
P_regen = Pbat;
P_regen(Pbat>= 0) = 0;
sol.E_discharge = cumtrapz(sol.time,P_discharge); %[J] Battery discharge energy
sol.E_regen     = cumtrapz(sol.time,P_regen    ); %[J] Battery regen energy

disp('=========== Energy Consumption Prediction has been completed succesfully. ============');

end
