function [dis,v_filt,acc] = filterVelocity( trk )
%FILTERVELOCITY - filters the velocityprofile contained in trk and
%additionally calculates the timederivative, i.e. the acceleration.
%
%   Syntax:
%       [dis,v_filt,acc] = FILTERVELOCITY( trk )
%
%   Description:
%       FILTERVELOCITY() - filters the velocity profile (velocity as
%       function of time) contained in 'trk' by applying a 3rd order
%       Butterworth low-pass filter. Subsequently, the filtered velocity is
%       differentiated w.r.t. time, resulting in the acceleration as
%       function of time.
%    
%   Inputs:
%       trk   - Struct containing the at least the fields:
%           time: [s] Equi-distant time vector
%            vel: [m/s] vehicle velocity
%
%   Outputs:
%       dis    - [m] Travelled distance, estimated basted on time-integration
%                of the filtered velocity
%       v_filt - [m/s] The low-pass filtered velocity
%       acc    - [m/s^2] The acceleration resulting fromt the low-pass
%                filtered velocity
%
%   Other m-files required: myButter, FiltFiltM (both developed by [1])
%   Subfunctions: none
%   MAT-files required: none
%
%   See also: myButter, FiltFiltM, diff, cumtrapz
%
%   [1] Jan Simon (2011). A faster FILTER and FILTFILT: Speedup factor 2.5 to 25
%   (https://nl.mathworks.com/matlabcentral/fileexchange/32261-filterm),
%   MATLAB Central File Exchange. Retrieved November 5, 2020.

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 18-Jul-2019; Last revision: 5-Nov-2020
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved.

%% Set parameters:
fc = 0.5; %[Hz] cutt-off frequency of velocity filter.

%% rename inputs
v = trk.vel;
n = length(v);
t = trk.time;

%determine sampling frequency:
dt = diff(t);
if any(abs(dt-dt(1))>1e-10); warning('time vector is not equidistant.');end

fs = 1/dt(1);  %[Hz] Sampling frequency
if fc>=fs/2; error('Chosen cut-off frequency is larger than Nyquist frequency. Please select lower cut-off frequency or incrase the sampling rate of the data.');end

%% Filter velocity
[b,a]=myButter(3,fc/(fs/2),'low'); %Butterworth low-pass filter of chosen frequency
v_filt= FiltFiltM(b,a,v);

%% Determine acceleration
acc = diff(v_filt)./dt; %[m/s^2] Acceleration based on filtered velocity
%Convention: acceleration determined between point n and n+1, is stored at
%index n in the acceleration vector.

dis = cumtrapz(t,v_filt); %[m] Traveled distance based on filtered velocity
end
