function bearing_comp = getCompassBearing( lat, lon )
%GETCOMPASSBEARING - Gives the compass bearing (e.g. the direction of the
%velocity vector w.r.t. north in [deg]) based on a GPS trace.
%
%   Syntax:
%       direction = GETCOMPASSBEARING(lat,lon)
%
%   Description:
%       GETCOMPASSBEARING() - Returns the direction of the vehicle relative
%       to the North as function of latitude and longitude coordinates.
%       When a vehicle is moving from South to North, the direction is 0.
%       The input has to be at least 2 latitude and 2 longitude
%       coordinates, but can also be 2 vectors of length N. The output
%       direction is than a vector of length N-1. The vector lat and vector
%       lon have to be of the same size.
%    
%   Inputs:
%       lat - array of doubles of size N indicating latitude coordinates[]
%       lon - array of doubles of size N indicating longitude coordinates 
%             []
%
%   Outputs:
%       bearing_comp - array of doubles  of size N-1 indicating compass
%       bearing [] w.r.t north:
%                      Vehicle driving towards north: bearing_comp = 0
%                      Vehicle driving towards east:  bearing_comp = 90
%                      Vehicle driving towards south: bearing_comp = 180
%                      Vehicle driving towards west:  bearing_comp = 270
%
%   Sources:
%   https://stackoverflow.com/questions/18520620/how-can-i-determine-direction-with-gps-readings-alone-and-display-in-360-degrees
%   http://www.movable-type.co.uk/scripts/latlong.html
%
%   Other m-files required: none
%   Subfunctions: none
%   MAT-files required: none
%
%   See also: 

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 25-Aug-2020; Last revision: 25-Aug-2020
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved.

%% Filter GPS coordinates:
%lon = smoothdata(lon,'gaussian',100);
%lat = smoothdata(lat,'gaussian',100);

%% Split data in sets:
lon1 = lon(1:end-1);
lon2 = lon(2:end);
lat1 = lat(1:end-1);
lat2 = lat(2:end);

%% Calculate bearing for each set of coordinates
dLon = lon2 - lon1;
y = sind(dLon) .* cosd(lat2);
x = cosd(lat1) .* sind(lat2) - sind(lat1) .* cosd(lat2) .*cosd(dLon);
bearing = atan2d(y, x);

% normalise the result to a compass bearing (0 - 360 deg)
bearing_comp  = mod(bearing+360,360);