function Curvature = getCornerCurvature( lat, lon, plotbool )
%GETCORNERCURVATURE - Determines the curvatures of a route, based on
%latitude/longitude coordinates.
%
%   Syntax:
%       Curvature = GETCORNERCURVATURE( lat, lon, plotbool )
%
%   Description:
%       GETCORNERCURVATURE() - Determines the curvature in [1/m] of a route,
%       based on the GPS coordinates based on 'lat' and 'lon'.
%       First the data is filtered using a Savitzky-Golay filter, after
%       which the corner radius is determined by determining the radius of
%       the circle circumscribing 3 consecutive gps coordinates.
%    
%   Inputs:
%       lat - array containing latitudes [deg]
%       lon - array containing longitues [deg]
%       plotbool - boolean indicating whether a plot should be generated or
%       not
%
%   Outputs:
%       Curvature - array containing the curvature in [1/m] for every
%       coordinate as defined by 'lat' and 'lon'.
%
%   Other m-files required: none
%   Subfunctions: Halversine_distance, Halversine_distance_skip
%   MAT-files required: none
%
%   See also: smoothdata

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 28-Aug-2019; Last revision: 01-Sep-2020
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved.

%% Set parameters.
SgolayW = 75;           %[m] Savitzky-Golay Window size
SgolayD = 5;            % Savitzky-Golay polynomial degree

%% Filder GPS data:
distance_raw = Halversine_distance(lat,lon);

%Filter GPS-data over non-uniform distance grid using Savitzky-Golay filter
lat_filt = smoothdata(lat,'sgolay',SgolayW,'Degree',SgolayD,'SamplePoints',distance_raw);
lon_filt = smoothdata(lon,'sgolay',SgolayW,'Degree',SgolayD,'SamplePoints',distance_raw);

%% Calculate curvature

%Determine distance to each next point and to the next second point
distance  = Halversine_distance(lat_filt,lon_filt);
dist_next = diff( distance );                   %[m] distance to next point
dist_skip = diff(Halversine_distance_skip(lat_filt,lon_filt));%[m] distance to second next point

%The curvature is calculated based on the location of the current point
%(b), the point before (a), and the point after (c). Together, these points
%span the triangle a-b-c, which is considered here:

%Define lengths of the sides of the triangle:
Lab = dist_next(1:end-1); %[m] length between a and b
Lbc = dist_next(2:end  ); %[m] length between b and c
Lac = dist_skip;          %[m] length between a and c

% Determine area of triangle using Heron's formula:
s    = 0.5*(Lab+Lbc+Lac);
Area = sqrt(s.*(s-Lab).*(s-Lbc).*(s-Lac+1e-11));

% Determine the curvature of the circle that touches the three points of
% the triangle:
Curvature = (4*Area)./(Lab.*Lbc.*Lac); %[1/m] curvature

%Padd curvature with NaN's as the curvature of the first and last point of
%the dataset can not be determined:
Curvature = [NaN; Curvature; NaN];

%% Determine the sign of the curvature
% left hand curve (positive) or right hand curve (negative)

%determine slope in (long,lat)-grid:
Slope_next   = (lat_filt(2:end)-lat_filt(1:end-1))./(lon_filt(2:end)-lon_filt(1:end-1)); %slope to next point
Slope_skip   = (lat_filt(3:end)-lat_filt(1:end-2))./(lon_filt(3:end)-lon_filt(1:end-2)); %slope to second next point

RightIndex = Slope_next(1:end-1)>Slope_skip(:);
Curvature(RightIndex) = -1*Curvature(RightIndex);

%% Plot the results (if plotbool == true)
if plotbool
    
    Plotrange = 1:numel(Curvature);
    
%     %plot curvature:
%     figure; box on; grid on;
%     plot(distance,Curvature);
%     xlabel('distance [m]');
%     ylabel('curvature [1/m]');
    
    %% Determine Center point of the circle that passes through the three
    
    %Coordinates (long,lat) of midpoints of lines between consecutive points
    LongMid = (lon_filt(1:end-1)+lon_filt(2:end))/2;
    LatMid  = (lat_filt(1:end-1)+lat_filt(2:end))/2;
    
    %Coordinates (lon,lat) of center of circle passing through consecutive
    %points
    CircLong = (Slope_next(1:end-1).*LongMid(2:end)-Slope_next(2:end).*LongMid(1:end-1)+Slope_next(1:end-1).*Slope_next(2:end).*(LatMid(2:end)-LatMid(1:end-1)))./(Slope_next(1:end-1)-Slope_next(2:end));
    CircLat  = (-1./Slope_next(1:end-1)).*(CircLong-LongMid(1:end-1))+LatMid(1:end-1);
    
    %Padd with NaN's, just as done with curvature vector above
    CircLong = [NaN; CircLong; NaN];
    CircLat  = [NaN; CircLat ; NaN];
    
    %plot raw GPS and filtered GPS signals:
    figure; hold on; axis equal
    title('Corner Curvature Plot')
    plot(lon(Plotrange),lat(Plotrange),':.r');
    plot(lon_filt(Plotrange),lat_filt(Plotrange),'.-','markersize',5);
    xlabel('Longitude [m]');
    ylabel('Latitude [m]');
    
    %For each of the datapoints
    for i = Plotrange
         
        %Determine inward direction of corner:
        Direction = [CircLong(i) - lon_filt(i) ; CircLat(i)-lat_filt(i)] ;
        DirectionNorm = Direction/norm(Direction); %make it a unitvector
        
        %Schale with the size of the found curvature
        PlotCurvature = -0.01*DirectionNorm*abs(Curvature(i));
        
        %Plot black line at outside of corner
        plot([lon_filt(i) lon_filt(i)+PlotCurvature(1)],[lat_filt(i) lat_filt(i)+PlotCurvature(2)],'k','Linewidth',0.5);

    end
    
    %legend('Filtered GPS route','Curvature \times10^3 [1/m]');
    
    axis equal
    
end

end %function

function distance = Halversine_distance(lat,lon)
%Input: latitude  vector [deg]
%       longitude vector [deg]

% Some local constants
r = 6371000; %[m] Earth Radius

lat1 = lat(1:end-1);
lat2 = lat(2:end);
lon1 = lon(1:end-1);
lon2 = lon(2:end);

%%
hav_lat = sind( (lat2-lat1)/2 ).^2;
hav_lon = sind( (lon2-lon1)/2 ).^2;

dis2next = 2*r*asin( sqrt( hav_lat + cosd(lat1).*cosd(lat2).*hav_lon ) );

distance = [0; cumsum(dis2next)];

end

function distance = Halversine_distance_skip(lat,lon)
%Input: latitude  vector [deg]
%       longitude vector [deg]

% Some local constants
r = 6371000; %[m] Earth Radius

lat1 = lat(1:end-2);
lat2 = lat(3:end);
lon1 = lon(1:end-2);
lon2 = lon(3:end);

%%
hav_lat = sind( (lat2-lat1)/2 ).^2;
hav_lon = sind( (lon2-lon1)/2 ).^2;

dis2next = 2*r*asin( sqrt( hav_lat + cosd(lat1).*cosd(lat2).*hav_lon ) );

distance = [0; cumsum(dis2next)];

end