function CumulativeDistance = getHalversineDistance( lat, lon )
%GETHALVERSINEDISTANCE - Determines the cumulative distance of a GPS trace
%by applying the Halversine formula.
%
%   Syntax:
%       CumulativeDistance = GETHALVERSINEDISTANCE( lat, lon )
%
%   Description:
%       GETHALVERSINEDISTANCE() - Calculates the distance between all the
%       consecutive GPS-points specified by 'lat' and 'lon' using the
%       Halversine formulate. A cumulative distance array is returned by
%       summing the distances inbetween the GPS-points.
%       This function assumes the earth is a perfect sphere, which it
%       isn't. A more accurate method to determine the cumulative distance
%       could be to use MATLAB's 'distance' function, which requires the
%       mapping toolbox.
%    
%   Inputs:
%       lat - array of doubles indicating latitude coordinates [deg]
%       lon - array of doubles indicating longitude coordinates [deg]
%
%   Outputs:
%       CumulativeDistance - array of doubles indicating the cumulative
%           distance for each point described by 'lat' and 'lon'. The
%           distance of the first point is assumed to be 0.
%
%   Other m-files required: none
%   Subfunctions: none
%   MAT-files required: none
%
%   See also: 

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 05-Nov-2020; Last revision: 05-Nov-2020
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved.

%% Define constants
r = 6371000; %[m] Earth Radius

%%
lat1 = lat(1:end-1);
lat2 = lat(2:end);
lon1 = lon(1:end-1);
lon2 = lon(2:end);

%%
hav_lat = sind( (lat2-lat1)/2 ).^2;
hav_lon = sind( (lon2-lon1)/2 ).^2;

dis2next = 2*r*asin( sqrt( hav_lat + cosd(lat1).*cosd(lat2).*hav_lon ) );

%%
CumulativeDistance = [0; cumsum(dis2next)];
end