function [rho, wind_speed, wind_dir] = getOWMweather(lat,lon,time)
%GETOWMWEATHER - Gives the airdensity, wind speed and wind direction at the
%current or forecast time at certain coordinates.
%
%   Syntax:
%       [rho, wind_speed, wind_dir] = GETOWMWEATHER(lat,lon,time)
%
%   Description:
%       GETOWMWEATHER() - Returns for time = -1 the current density, wind
%       speed and wind direction based on latitude and longitude
%       coordinates. For a time > 0 it gets information of OpenWeatherMap
%       for the estimated forecast time at 'time'. This also works for
%       vector inputs, but lat and lon must be of the same size. For
%       forecast weather information, also time should be of the same size
%       of lat and lon, or should be one value to get the weather
%       information at a steady forecast time. This is however the same
%       when the input of time is a vector the same number.
%    
%   Inputs:
%       lon  - Longitude coordinates []
%       lat  - Latitude coordinates []
%       time - Time after current time when you want to know the weather
%              conditions [seconds] or -1 for the current weather
%              conditions. The prediction is limited to 5 days in advance,
%              therefor the max value 'time' can have is: 5*24*3600 =
%              432000.
%
%   Outputs:
%       rho        - Density of the air [kg/m3]
%       wind_speed - Wind speed [m/s]
%       wind_dir   - Wind direction []
%
%   Other m-files required: getAirDensity
%   Subfunctions: datenum_to_unixtime
%   MAT-files required: none
%
%   See also: getAirDensity
%
%   References:
%   [1] Jose Luis Prego (2020). Air density calculation
%   (https://www.mathworks.com/matlabcentral/fileexchange/12565-air-density-calculation),
%   MATLAB Central File Exchange. (getAirDensity.m)
%   [2] https://openweathermap.org/current

%   Author: Tim Geraedts
%   email: t.a.g.h.geraedts@student.tue.nl
%   Date: 25 February 2019; Last revision: 27 Oct 2020
%
%   Copyright (c) 2020, T.A.G.H.Geraedts
%   All rights reserved.

%% Calculation

%Read API-key:
fid = fopen('OpenWeatherMap_key.txt');
key = textscan(fid,'%s');
fclose(fid);
key = key{1}{1};

resolution=1000; % Increasing this number means that the difference in 
 % latitude and longitude coordinates becomes smaller, and more information
 % is gotten. This however increases the total running time.
 % The value 1000 is chosen as this is in difference a length of about 10m.

% This reference will probably not working after 2038 as the 
% 'YEAR 2038 PROBLEM' will occur. However, free account doesn't have
% historic data access, so this does not give problems.
% history = ['http://history.openweathermap.org/data/2.5/history/city?',...
%         'lat=',num2str(lat(1)),'&lon=',num2str(lon(1)),'&type=', ...
%         'hour&start=',num2str(time(1)),'&end=',num2str(time(end))];

length_loop=length(lat);
rho=zeros(1,length_loop);
wind_speed=zeros(1,length_loop);
wind_dir=zeros(1,length_loop);

%% Get current weather information
if time == -1 % Weathermap is loaded on current time

    for i=1:length_loop
        if (i>1 && round(lat(i)*resolution)==round(lat(i-1)*resolution) && ...
                round(lon(i)*resolution)==round(lon(i-1)*resolution))
            rho(i) = rho(i-1);
            wind_speed(i) = wind_speed(i-1);
            wind_dir(i) = wind_dir(i-1);
        else
            current = ['https://api.openweathermap.org/data/2.5/weather?lat=', ...
                      num2str(lat(i),8),'&lon=',num2str(lon(i),8)];
            url = [current,'&APPID=',key];

            Current_Data = webread(url , weboptions('Timeout',25)); % Max
            %  25 s before it is chosen that getting info takes too long

            main = Current_Data.main;

            rho(i) = getAirDensity( main.temp-273.15 , main.humidity , ...
                main.pressure*100);

            wind = Current_Data.wind;
            wind_speed(i) = wind.speed;
            if (length(fieldnames(wind))~=1)
                wind_dir(i) = wind.deg;
            else
                wind_dir(i) = nan;
            end
        end
    end

else % Weathermap is loaded on estimated time, provided in 'time'
    
%% Get forecast weather information
    if length(time)==1
       time = ones(1,length_loop)*time;
    else
        if length(time)~=length_loop
           disp('Size of time is not the same as coordinates size')
           return
        end
    end

    %https://openweathermap.org/forecast5
    for i=1:length_loop
        if (i>1 && round(lat(i)*resolution)==round(lat(i-1)*resolution) ...
                && round(lon(i)*resolution)==round(lon(i-1)*resolution))
            
            ileft  = find(forecast_time <= time(i));
            ileft = ileft(end);
            iright = find(forecast_time >  time(i),1);
            if isempty(iright)
               iright = length(forecast_time); 
            end
            frac = (time(i) - forecast_time(ileft)) / ...
                (forecast_time(iright) - forecast_time(ileft));

            rho(i) = frac * (arho(iright) - arho(ileft)) + arho(ileft);
            wind_speed(i) = frac * (awind_speed(iright) - ...
                awind_speed(ileft))    + awind_speed(ileft);
            if isnan(awind_dir(ileft))
                if isnan(awind_dir(iright))
                    wind_dir(i) = nan;
                else
                    wind_dir(i) = awind_dir(iright);
                end
            else
                if isnan(awind_dir(iright))
                    wind_dir(i) = awind_dir(ileft);
                else
                    wind_dir(i) = frac * (awind_dir(iright) - ...
                        awind_dir(ileft))  + awind_dir(ileft);
                end
            end
            
        else % Resolution is too low, new data should be called
            forecast = ['https://api.openweathermap.org/data/2.5/forecast?lat=', ...
                      num2str(lat(i),8),'&lon=',num2str(lon(i),8)];
            url = [forecast,'&APPID=',key];

            Forecast_Data = webread(url , weboptions('Timeout',25)); % Max
            % 25 s before it is chosen that getting info takes too long

            Data = Forecast_Data.list;
            l_data = length(Data);
            forecast_time=zeros(1,l_data);
            arho=zeros(1,l_data);
            awind_speed=zeros(1,l_data);
            awind_dir=zeros(1,l_data);

            for a=1 : l_data % Get all forecast data
                try % Sometimes it needs to be {} and sometimes ()
                    forecast_time(a) = Data{a}.dt;
                    awind_speed(a) = Data{a}.wind.speed;
                    if (length(fieldnames(Data{a}.wind))~=1)
                        awind_dir(a) = Data{a}.wind.deg;
                    else
                        awind_dir(a) = nan;
                    end
                    arho(a) = getAirDensity(Data{a}.main.temp-273.15 , ...
                        Data{a}.main.humidity , Data{a}.main.pressure*100);
                catch
                    forecast_time(a) = Data(a).dt;
                    awind_speed(a) = Data(a).wind.speed;
                    if (length(fieldnames(Data(a).wind))~=1)
                        awind_dir(a) = Data(a).wind.deg;
                    else
                        awind_dir(a) = nan;
                    end
                    arho(a) = getAirDensity(Data(a).main.temp-273.15 , ...
                        Data(a).main.humidity , Data(a).main.pressure*100);
                end
            end
            
            
            current = ['https://api.openweathermap.org/data/2.5/weather?lat=', ...
              num2str(lat(i),8),'&lon=',num2str(lon(i),8)];
            url = [current,'&APPID=',key];

            Current_Data = webread(url , weboptions('Timeout',25)); % Max
            % 25 s before it is chosen that retrieving info takes too long

            if (Current_Data.dt < forecast_time(1)) % The current time is
                                 % older then first entry of forecast time
                forecast_time = [Current_Data.dt  forecast_time];

                current_rho=getAirDensity(Current_Data.main.temp-273.15, ...
                   Current_Data.main.humidity, Current_Data.main.pressure*100);
                arho = [current_rho arho];

                awind_speed = [Current_Data.wind.speed  awind_speed];
                if (length(fieldnames(Current_Data.wind))~=1)
                    awind_dir = [Current_Data.wind.deg  awind_dir];
                else
                    awind_dir = [nan awind_dir];
                end
            end
            
            
            if i==1 % First time the 'time' should be made
                now_forecast = (datenum_to_unixtime(datenum(datetime('now','TimeZone','UTC'))) - forecast_time(1));
                time = time + now_forecast;
                % datetime(ans, 'ConvertFrom', 'posixtime')
                % For example: it is right now 18:29. Forecast_time(1)
                % is 18:28. You want to now exactly the data now, so
                % time = 0; If forecast_time below is made 0, you get
                % the time at 18:28. But you want to know the time at
                % 18:29, so time = time + 1 minute is made, and the
                % data is achieved at 18:29.
            end
            
            forecast_time=forecast_time-forecast_time(1); % [seconds] with 
                            % starting time 0 seconds ( forecast_time(1) )

            ileft  = find(forecast_time <= time(i));
            ileft = ileft(end);
            iright = find(forecast_time >  time(i),1);
            if isempty(iright)
               iright = length(forecast_time); 
            end
            frac = (time(i) - forecast_time(ileft)) / ...
                (forecast_time(iright) - forecast_time(ileft));

            rho(i) = frac * (arho(iright) - arho(ileft)) + arho(ileft);
            wind_speed(i) = frac* (awind_speed(iright) - awind_speed(ileft))...
                + awind_speed(ileft);
            if isnan(awind_dir(ileft))
                if isnan(awind_dir(iright))
                    wind_dir(i) = nan;
                else
                    wind_dir(i) = awind_dir(iright);
                end
            else
                if isnan(awind_dir(iright))
                    wind_dir(i) = awind_dir(ileft);
                else
                    wind_dir(i) = frac * (awind_dir(iright) - ...
                        awind_dir(ileft))  + awind_dir(ileft);
                end
            end
        end
    end
end
end

function ut = datenum_to_unixtime( dn )
    ut = (dn - 719529)*86400;              %# == datenum(1970,1,1)
end