function [Mag_W,beta,rho] = getRelativeWind( trk, windfactor, plotbool )
%GETRELATIVEWIND - Returns the magnitude and angle of the wind RELATIVE to
%the moving vehicle, which is described by the struct 'trk'. The
%calculations, including those of the air density 'rho', depend on
%up-to-date weather information from OpenWeatherMap.
%
%   Syntax:
%       [Mag_W,beta,rho] = GETRELATIVEWIND( trk, windfactor, plotbool )
%
%   Description:
%       GETRELATIVEWIND() - Returns the magnitude 'Mag_W' and angle 'beta'
%       of the wind RELATIVE to the moving vehicle. The bearing and
%       velocity of the vehicle is calculated from the information included
%       in 'trk'. 
%       The magnitude of the vehicle velocity vector is based on the
%       velocity 'trk.vel', while the bearing is calculated from the GPS
%       coordinates 'trk.lat' and 'trk.lon'.
%       The absolute wind direction and magnitude is taken from
%       OpenWeatherMap, after which the relative wind magnitude and
%       direction are determined. For extact definitions, a refernce to [1]
%       is made.
%    
%   Inputs:
%       trk - Struct containing the at least the fields:
%           trk.vel: [m/s] array indicating the vehicle velocity
%           trk.lat: [deg] array with latitude of coordinates
%           trk.lon: [deg] array with longitude of coordinates
%       windfactor - [-]Factor that is multiplied with the absolute wind
%           magnitude before including it in the calculations. This factor
%           corrects for the fact that the absolute wind magnitude is
%           measured at an altitude of 10m, whereas the vehicle drives on
%           ground level.
%       plotbool - Boolean indicating whether to generate a plot or not
%
%   Outputs:
%       Mag_W - [m/s] Magnitud of the relative wind velocity 
%       beta - [deg] Angle of the relative wind vector w.r.t. the
%           longitudinal vehicle direction, according to the definition
%           below:
%
%                       beta
%               long.  |_+_/ headwind direction
%         vehicle dir. |  /
%                     _|_/
%                    | |/|
%                    | . | vehicle
%                    |___|
%   
%   beta = 0  [deg] headwind
%   beta = 90 [deg] relative wind exactly from the right
%   beta =-90 [deg] relative wind exactly from the left
%   beta = 180[deg] tailwind
%
%   Other m-files required: getOWMweather, getCompassBearing
%   Subfunctions: getAngleAveraged, wrapAngle360, wrapAngle180
%   MAT-files required: none
%
%   See also: getOWMweather, getCompassBearing
%
%   [1] C.J.J. Beckers, M. Paroha, I.J.M. Besselink, and H. Nijmeijer, "A
%   Microscopic Energy Consumption Prediction Tool for Fully Electric
%   Delivery Vans," in EVS33 International Battery, Hybrid and Fuel Cell
%   Electric Vehicle Symposium, 2020.

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 06-Aug-2019; Last revision: 22-Dec-2020
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved.

%%
[rho,wind_speed,dir_wind] = getOWMweather(mean(trk.lat,'omitnan'),mean(trk.lon,'omitnan'),-1); % No use of forecast
rho = mean(rho);
%dir_wind = direction the wind is moving FROM (360 = FROM north, 90 =
%FROM east, 180 FROM south, 270 = FROM west).

%Wind velocity vector direction (w.r.t. north):
Psi_W = wrapAngle360(mean(dir_wind)-180); %[deg] direction wind is blowing TOWARDS w.r.t north
vel_W = windfactor*mean(wind_speed); %[m/s] wind velocity
vec_W = vel_W* [ sind(Psi_W); cosd(Psi_W) ; 0 ];

Psi_V = getCompassBearing(trk.lat,trk.lon)';%direction vehicle is moving TOWARDS w.r.t. north
%0 = TOWARDS north, 90 = TOWARDS east, 180 = TOWARDS south, 270 = TOWARDS west
Psi_V_avg = wrapAngle360(getAngleAveraged(Psi_V(~isnan(Psi_V))));
vel_V = trk.vel(1:end-1)+0.5*diff(trk.vel); %[m/s] vehicle velocity
vec_V = [vel_V'.*sind(Psi_V) ; vel_V'.*cosd(Psi_V) ; zeros(size(Psi_V))];

%Determine relative wind velocity vector
vec_A = vec_W-vec_V;
Mag_W = sqrt(sum(vec_A.^2));
beta  = wrapAngle180(-Psi_V+atan2d(vec_A(1,:),vec_A(2,:))+180); %[deg] headwind angle with respect to vehicle

%% Plot
if plotbool
    figure;
    compass(mean(trk.vel)*exp(Psi_V_avg*(2*pi/360)*(sqrt(-1))),'r'); hold on;
    compass(mean(vel_W/windfactor)*exp(  Psi_W  *(2*pi/360)*(sqrt(-1))))
    camroll(90);view([90 -90]); %Switch axis to be in accordance with wind direction definitions
    legend('Average Vehicle Velocity [m/s]','Average Wind Velocity [m/s]');
    title('Wind Direction Plot')
end
end

function alpha = getAngleAveraged(A)
% calculate average angle:
n = numel(A);
alpha = atan2(1/n*sum(sind(A)), 1/n*sum(cosd(A)))/(2*pi)*360;
end

function alpha = wrapAngle360(alpha)
%Wraps angle to be in [0,360].
index_small = 1;
index_large = 1;
while any(index_small) || any(index_large)
    alpha(index_small) = alpha(index_small)+360;
    alpha(index_large) = alpha(index_large)-360;
    
    index_small = alpha < 0;
    index_large = alpha > 360;
end
end

function alpha = wrapAngle180(alpha)
%Wraps angle to be in [-180,180].
index_small = 1;
index_large = 1;
while any(index_small) || any(index_large)
    alpha(index_small) = alpha(index_small)+360;
    alpha(index_large) = alpha(index_large)-360;
    
    index_small = alpha < -180;
    index_large = alpha > 180;
end
end