function [elev, elevrawp, grad] = getSRTMelevation( trk, dataFolder, plotbool )
%GETSRTMELEVATION - returns the elevation of a given route, based on the
% SRTM elevenation map.
%
%   Syntax:
%       [elev, grad, elevrawp] = GETSRTMELEVATION( trk, dataFolder, plotbool )
%
%   Description:
%       GETSRTMELEVATION() - Returns the elevation profile of the route
%       defined by the route defined in 'trk'. The elevation is based on
%       the Shuttle Radar Topography Mission (SRTM) elevenation map. The
%       elevation is queried for each of the coordinates defined in trk.
%       The elevation profile is filtered before it is returned. A 3rd
%       order Butterworth low-pass filter is used to this end. The raw
%       elevation profile is also returned as 'elevrawp'.
%       The function also returns the road gradient in [rad]. This profile
%       is determined by differentiating the filtered elevation profile
%       with respect to the travelled distance.
%    
%   Inputs:
%       trk - Struct containing the at least the fields:
%           trk.dis: [m]  array indicating the travelled distance
%           trk.lat: [deg] array with latitude of coordinates
%           trk.lon: [deg] array with longitude of coordinates
%       dataFolder - string defining the folder where the SRTM map data is
%       to be stored
%       plotbool - boolean indicating whether to plot the elevation map or
%       not
%
%   Outputs:
%       elev - [m] array describing the filtered elevation at the requested
%              coordiantes
%       elevrawp - [m] array describing the raw elevation data at the
%                  requested coordinates
%       grad - [rad] array descrig the radient between the reqested
%              coordinates
%
%   Other m-files required: readhgt (developed by [1]), myButter, FiltFiltM
%   (developed by [2])
%   Subfunctions: none
%   MAT-files required: none
%
%   See also: readhgt, myButter, FiltFiltM
%
%   [1] Franois Beauducel (2020). READHGT: Import/download NASA SRTM data
%   files (.HGT)
%   (https://www.mathworks.com/matlabcentral/fileexchange/36379-readhgt-import-download-nasa-srtm-data-files-hgt),
%   MATLAB Central File Exchange. Retrieved August 31, 2020. 
%
%   [2] Jan Simon (2011). A faster FILTER and FILTFILT: Speedup factor 2.5 to 25
%   (https://nl.mathworks.com/matlabcentral/fileexchange/32261-filterm),
%   MATLAB Central File Exchange. Retrieved November 5, 2020.

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 26-Jul-2019; Last revision: 5-Nov-2020
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved.

%% Set parameters
stepsize = 10;         % [m]  stepsize of dense distance grid. 
lambda_s = 1/stepsize;%[1/m] Spatial sampling frequency of dense distance gridChanging this value does not have a lot of influence, as long as it is sufficiently smaller than the cut-off frequency
lambda_c = 1/2000;   %[1/m] Spatial cut-off frequency of low-pass altitude filter

%% PRE-PROCESSING
%Find GPS values that are NaN
nanIndex    = isnan(trk.lon) | isnan(trk.lat);
n           = numel(~nanIndex); %number of real data points

%For this function: only consider values that are not nan:
lon = trk.lon(~nanIndex);
lat = trk.lat(~nanIndex);
dis = trk.dis(~nanIndex);

%Initialize output variable:
elev    = zeros(size(trk.lon));
elevrawp= zeros(size(trk.lon));
grad    = zeros(size(elev(1:end-1)));

%GPS values that are nan, also have height nan:
elev(nanIndex,1) = NaN;
elevrawp(nanIndex,1) = NaN;
if numel(nanIndex) > numel(grad)
    grad(nanIndex(1:end-1),1) = NaN;
else
    grad(nanIndex(1:end),1) = NaN;
end

%% Download required tiles from SRTM:
if plotbool %If requied, make plot
    map = readhgt(unique(floor(lat)),unique(floor(lon)),'interp','outdir',dataFolder ,'plot');
    
    %Plot additional GPS trace on map:
    hold on;
    ax = plot(lon,lat,'r');
    legend(ax,'GPS trace'); clear ax;
else %If plotting is not desired:
    %Collect map data from SRTM:
    map = readhgt(unique(floor(lat)),unique(floor(lon)),'interp','outdir',dataFolder);
end

%Interpolate 2D map data to obtain height information along 1D GPS-trace:
elevraw = interp2(map.lon,map.lat,double(map.z),lon,lat);
elevrawp(~nanIndex) = interp2(map.lon,map.lat,double(map.z),lon,lat);

%% Filter altitude

%Define dense, uniform distance grid
disgrid    = 0:stepsize:dis(end);

%Sort distance and altitude values:
[dis_sort,SortIndex] = sort(dis);
elevraw = elevraw(SortIndex);

%Identify values from the distance grid that are not unique:
[~,UniIndex] = unique(dis_sort);
NotUniIndex = ~ismember(1:numel(dis_sort),UniIndex);
%And eliminate these fromt he data:
dis_sort(NotUniIndex) = [];
elevraw(NotUniIndex) = [];

%Interpolate altitude values to the uniform (= equidistant) distance grid
elevgridraw = interp1(dis_sort,elevraw,disgrid,'linear');

%Apply a low-pass filter in the distance domain:
[b,a] = myButter(3,lambda_c/(lambda_s/2),'low'); %Butterworth low-pass filter of chosen spatial frequency
elevgridfilt = FiltFiltM(b,a,elevgridraw);

%Determine the gradient corresponding to the filtered altitude
gradgrid = atan(diff(elevgridfilt)./diff(disgrid));  %[rad] gradient based on filtered altitude profile
                                                    %Convention: gradient determined between point n and n+1, is stored at
                                                    %index n in the gradient vector.

%Interpolate altitude and gradient to match with the original data points
elev(~nanIndex)  = interp1(disgrid,elevgridfilt,dis,'linear',NaN);
grad(~nanIndex) = interp1(disgrid(1:end-1)+stepsize/2,gradgrid,dis,'linear',0);
if numel(grad) == numel(elev)
    grad(end) = [];
end
    
end
