function trk = processOSMmap(route,openstreetmap_filename,plotbool)
%PROCESSOSMMAP - Extracts data from an OpenStreetMap file for a particular
%route.
%
%   Syntax:
%       trk = PROCESSOSMMAP(route,openstreetmap_filename)
%
%   Description:
%       PROCESSOSMMAP() - Opens and parces the OpenStreetMap-file specified
%       by 'openstreetmap_filename' and extracts relevant information, but
%       only for the route specified by 'route.
%       1. To this end, the .osm-file specified by 'openstreetmap_filename',
%       which is in .xml-format, is parsed into a struct, which is then
%       stored in a .mat-file and stored in the same folder as the original
%       .osm-file. 
%       2. In the second part of this function, the resulting struct is
%       matched to the route specified by the coordinates in 'route', by
%       finding the nodes in the map closest to each of the coordinates of
%       the route.
%       3. Lastly, relevant information, suchs as maximum legislated
%       velocity and traffic sign location, is extracted from the map, but
%       only for the ways connecting the coordinates of the route. This
%       results in a struct 'trk', which contains relevant map information
%       for the requested route.
%    
%   Inputs:
%       route - A struct containing at least the following fields:
%           route.lat - Array of doubles indicating latitudes [deg]
%           route.lon - Array of doubles indicatings longitudes [deg]
%       Together, route.lat and route.lon describe the coordinates of a
%       route.
%       openstreetmap_filename - String specifying the path to the
%       .osm-file that contains map information. The map described by the
%       specified file should cover the same area as specified by the route
%       specified by the coordinates in 'route'.
%       plotbool - boolean indicating whether a plot will be generated or
%       not
%
%   Outputs:
%       trk - Struct describing more detailed information about the the
%             route. Trk has the following fields:
%           lon - [deg] Longitude coordinates
%           lat - [deg] Latitude coordinates
%           dis - [m] Travelled distance 
%           wayInfo - struct containing all
%           information that OpenStreetMap has on the requested route.
%           Therefore, this struct can have many fields, but at least
%           should include. 
%               maxspeed - [kmh] maximum legislated velocity
%               traffsign - boolean array indiating whether there is a traffic
%               sign on any of the requested route coordinates.
%
%   Other m-files required: parse_openstreetmap (requires [1]),
%   getHalversineDistance
%   Subfunctions: none
%   Other files required: OpenStreetMapFunctions, as described in [1],
%   should be part of the current working directory.
%
%   See also: parse_openstreetmap, getHalversineDistance
%
%   [1]  Ioannis Filippidis. Openstreetmap functions
%   (https://github.com/johnyf/openstreetmap) Github, 2016. 
%   [Retrieved Aug 31, 2020].

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 26-Aug-2019; Last revision: 22-Dec-2020
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved.

%% 1. convert XML -> MATLAB struct
% convert the OpenStreetMap XML Data file donwloaded as map.osm
% to a MATLAB structure containing part of the information describing the
% transportation network

parsedMapFile = [openstreetmap_filename(1:end-4) '.mat'];

if exist(parsedMapFile,'file')
    
    %File already exists. load data from file
    load(parsedMapFile);
    if ~exist('mapmatchbool','var') %Add mapmatchbool, if not present
        mapmatchbool = false;
    end
    
    disp(['Parsed map was found on local disk: ' parsedMapFile '']);
    
else %file does not exist parse osm data (
    if ~exist(openstreetmap_filename,'file')
        
        error(['OSM map-file ' openstreetmap_filename 'not found.']);
        
    else
        disp('Parsing map data. This might take several minutes...');

        [parsed_osm, osm_xml] = parse_openstreetmap(openstreetmap_filename);
        save(parsedMapFile,'parsed_osm','osm_xml');
        
        %Parsed map has just been calculated, so mapmatching will still
        %need to be approved:
        mapmatchbool = false;
        
        disp(['Parsing finished. Parsed map saved as: ' parsedMapFile '.']);
        
    end
    
end

%% 2. Find Node ID's corresponding to route

%Identify indices of notes that are closest to each of the route points:
[minDist,index] = min(((route.lat-parsed_osm.node.xy(2,:)').^2+(route.lon-parsed_osm.node.xy(1,:)').^2));

% ========= beginning of commented section ========= %
% The following section of commented code can be used to let the user
% explicitly approve the map-matching results for any new route. Once a
% route has been approved by the user once, this this remembered, untill
% the route or the map data is updated.
% In the current version of the code, this section is commented out,
% implying that any map-matching result is approved without user
% interaction.
%{ 
misMatchIndex = unique(index(minDist > 1e-10));
if ~isempty(misMatchIndex) && mapmatchbool == false %If there is at least one point that shows a large deviation:
    disp(['Map-matching: There are points of the route that do not exactly match OpenStreetMap.' newline...
          'This can happen when the route is older and therefore differs from the (updated) map.' newline...
          'In this case there are ' num2str(numel(misMatchIndex)) ' points that show a (small) deviation.']);
    
    %Plot result for visual check
    figure; hold on;
    plot(route.lon,route.lat)
    plot(parsed_osm.node.xy(1,        index)',parsed_osm.node.xy(2,        index)',':k')
    plot(parsed_osm.node.xy(1,misMatchIndex)',parsed_osm.node.xy(2,misMatchIndex)','*r')
    xlabel('longitude [deg]');
    ylabel('latitude [deg]');
    legend('original route (.gpx-file)','route matched to OpenStreetMap','Possible mismatch','location','best');
    
    %Let user decide whether to continue or stop the script:
    prompt = ['Please inspect the provided plot. ' newline...
        '               Do you want to continue with the current route and map-matching? Y/N: '];
    mapmatchbool = [];
    while isempty(mapmatchbool) %while no answer has been defined:
        str = input(prompt,'s'); %Prompt user
        if strcmpi(str,'Y') %If user says 'yes':
            mapmatchbool = true;
            save(parsedMapFile,'mapmatchbool','-append')
            break;
        elseif strcmpi(str,'N') %If user says 'no':
            mapmatchbool = false;
            error('The script has been stopped because the route matched to OpenStreetMap was not approved by the user.');
        end
    end
    
end
%} 
% ========= end of commented section ========= %

%Remove repeated occurences of nodes:
index(index(1:end-1) == index(2:end)) = [];

% Determine distance vector of route:
trk.lon = parsed_osm.node.xy(1,index)';
trk.lat = parsed_osm.node.xy(2,index)';
trk.dis = getHalversineDistance(trk.lat,trk.lon);

%% 3.1 Loop over ways to detect maximum velocity
%Specify waitbar details:
waitbarspaces = 38;
waitbarcounter = 1;
waitbar = ['[' repmat(' ',1,waitbarspaces) ']'];
disp(['Extracting route information from parsed map: ' waitbar ]);

N = numel(index)-1; %loop over sets of connecting nodes
for i = 1:N
    
    %Print waitbar to command window:
    if i >= N*waitbarcounter/waitbarspaces
        waitbarcounter = waitbarcounter + 1;
        waitbar(waitbarcounter) = '#';
        fprintf([repmat(char(8),1,numel(waitbar)+1) waitbar newline]);
    end
    
    nodeID1 = parsed_osm.node.id(1,index(i))  ; %ID of node 1
    nodeID2 = parsed_osm.node.id(1,index(i+1)); %ID of node 2
    
    %find ways that link each set of two connecting nodes
    foundway      = 0; %no way connecting the nodes has been found yet
    foundwayIndex = 0;
    doubleway     = false;
    
    for j = 1:numel(parsed_osm.way.id) %loop over all ways
        
        nodes_in_way = parsed_osm.way.nd{j}; %nodes in the considered way
        
        %If both the considered nodes are part of the considered way:
        if any(nodeID1 == nodes_in_way) && any(nodeID2 == nodes_in_way)
                
            foundway = foundway + 1; %mark that a way has been found
            foundwayIndex = j;       %mark index of found way
            foundspeed = 0;          %speed found boolean
            
            nFields = numel(parsed_osm.way.tag{j}); %number of info-fields
            
            if nFields == 1 %if there is only one-info field:
                fieldname = parsed_osm.way.tag{j}.Attributes.k;
                fieldname(regexp(fieldname,'[^\w]'))=[];
                content = parsed_osm.way.tag{j}.Attributes.v;
                trk.wayInfo(i,1).(fieldname) = content;
            else %there are multiple info fields:
                for k = 1:numel(parsed_osm.way.tag{j}) %loop over info fields
                    %store fieldname:
                    fieldname = parsed_osm.way.tag{j}{k}.Attributes.k;
                    fieldname(regexp(fieldname,'[^\w]'))=[];
                    
                    %store content of info-field
                    content = parsed_osm.way.tag{j}{k}.Attributes.v;
                    
                    %If the info is a velocity, convert to couble:
                    if strcmp(fieldname,'maxspeed')
                        foundspeed = 1; %Indicate that a speed has been found
                        
                        % read maxspeed (taking int account the unit)
                        [content] = textscan(content,'%f %s','Delimiter',' ');
                        if ~isempty(content{2})
                            unit = content{2}{1};
                        else
                            unit = '';
                        end
                        content = content{1};
                        
                        %Apply unit conversion to km/h, if nessecary:
                        if strcmpi(unit,'mph')
                            content = content*1.609344;
                        elseif strcmpi(unit,'knots')
                            content = content*1.852;   
                        end
                       
                    end
                    
                    if foundway > 1 && foundspeed == 0 %Multiple ways have been found that connect these two nodes but the most recent one does not contain speed info
                        % do not store most recent info
                        doubleway = true;
                    else                                           
                    %Store info in the trk struct:
                    trk.wayInfo(i,1).(fieldname) = content;
                    end
                    
                end %loop over info fields
            end %Info field analysis
            
            %If no field containing the speed has been found, mark as NaN:
            if foundspeed == 0 && doubleway == false
                trk.wayInfo(i,1).maxspeed = NaN;
            end
            
        end
        
    end
    
    if foundway == 0
        %warning(['Connection between nodes ' num2str(nodeID1) ' and ' num2str(nodeID2) ' are not part of any way.' newline 'The information from the previous found way will be assumed.'])
        if isfield(trk,'wayInfo')
            trk.wayInfo(i,1) = trk.wayInfo(i-1,1); %assume info of previous piece of track
        else
            trk.wayInfo(i,1).maxspeed = NaN; %if no previous piece of track is known, assume nothing.
        end
    end
    
end

%% 3.2 Finding the traffic-sign nodes
traffic_sign_attr = {'traffic_signals', 'traffic_sign'};

trafficsign_tags=cell(1,1);
trafficsign_tags_nodes=[];
%trafficsign_lat=[]; % This can be added, but now the traffic sign is found
%trafficsign_lon=[]; %  by comparing the ids of the road and traffic sign
a=0;
count = 0;
for i=index
    count = count+1;
    trafficsignal = false;
    try
        osm_xml.node{i}.tag;
        
        if length(osm_xml.node{i}.tag) == 1
            b = ismember(osm_xml.node{i}.tag.Attributes.k,traffic_sign_attr) ...
                || ismember(osm_xml.node{i}.tag.Attributes.v,traffic_sign_attr);
        else
            for j=1:length(osm_xml.node{i}.tag)
                b = ismember(osm_xml.node{i}.tag{j}.Attributes.k,traffic_sign_attr) ...
                || ismember(osm_xml.node{i}.tag{j}.Attributes.v,traffic_sign_attr);
                if b
                   break; 
                end
            end
        end
         
        if b
            a=a+1;
            %trafficsign_lat=[trafficsign_lat str2double(osm_xml.node{i}.Attributes.lat)];
            %trafficsign_lon=[trafficsign_lon str2double(osm_xml.node{i}.Attributes.lon)];
            if length(osm_xml.node{i}.tag)==1
                trafficsign_tags{a,1} = strcat(osm_xml.node{i}.tag.Attributes.k,',',osm_xml.node{i}.tag.Attributes.v);
                trafficsign_tags_nodes=[trafficsign_tags_nodes i];
                trafficsignal = true;
            else
                for j=1:length(osm_xml.node{i}.tag)
                    trafficsign_tags{a,j} = strcat(osm_xml.node{i}.tag{j}.Attributes.k,',',osm_xml.node{i}.tag{j}.Attributes.v);
                end
                trafficsign_tags_nodes=[trafficsign_tags_nodes i];
                trafficsignal = true;
            end
        end
    end
    trk.traffsign(count,1) = trafficsignal;
end

%% Plot the route
if plotbool
    cmap_vel = parula(max(50,ceil(max([trk.wayInfo.maxspeed]))));
    figure; hold on;
    for i = 1:numel(trk.wayInfo)
        current_vel = trk.wayInfo(i).maxspeed;
        if isnan(current_vel)
            h1 = plot([trk.lon(i)'; trk.lon(i+1)'],[trk.lat(i)'; trk.lat(i+1)']','k--');
        else
            color = cmap_vel(round(trk.wayInfo(i).maxspeed),:);
            plot([trk.lon(i)'; trk.lon(i+1)'],[trk.lat(i)'; trk.lat(i+1)'],'color',color);
        end
    end
    axis([min(trk.lon)-0.01 max(trk.lon)+0.01 min(trk.lat)-0.01 max(trk.lat)+0.01]);
    daspect([1 1 1]);
    xlabel('Longitude [deg]');
    ylabel('Latitude [deg]');
    c = colorbar('Ticks',[0, 30,50,80,100,120,130]/max([trk.wayInfo.maxspeed]),...
        'TickLabels',{'0','30','50','80','100','120','130'});
    c.Label.String = 'Legislated speed limit [km/h]';
    
    h2 = plot(parsed_osm.node.xy(1,trafficsign_tags_nodes),parsed_osm.node.xy(2,trafficsign_tags_nodes),'ok');
    if ~isempty(h2) && exist('h1','var')
        legend([h2 h1],'Traffic light','Legislated speed limit unknown','location','best');
    elseif exist('h1','var')
        legend(h1,'Legislated speed limit unknown','location','best');
    elseif ~isempty(h2)
        legend(h2,'Traffic light','location','best');
    end
    title('OpenStreetMap Route Data');
end

end
