function [lat,lon] = readGPXfile( fileName, mapTBpresent, maxDistance )
%READGPXFILE - Reads the specified GPX-file and outputs the latitudes and
%longitudes of this file in vectors.
%
%   Syntax:
%       [lat, lon] = READGPXFILE( fileName, MapTBpresent, maxDistance )
%
%   Description:
%       READGPXFILE() - Reads the .gpx-file specified by 'fileName' and
%       gives the latitude and longitudes of the coordinates as outputs.
%       Secondly, the file is checked for two criteria:
%           1. Whether the file has been generated using an
%           OpenStreetMap-based routing service.
%           2. Whether the route specified in the file is shorter than
%           'maxDistance'.
%       If these criteria are not met, appropriate warnings/errors are
%       thrown.
%    
%   Inputs:
%       fileName - string specifying the full path and file name of the
%           .gpx file that is to be analysed.
%       mapTBpresent - a boolean indicating whether the MATLAB mapping
%           toolbox is installed on the system.
%       maxDistance - double specifing the maximum allowable route distance
%           in [m].
%
%   Outputs:
%       lat - row vector of doubles specifying latitudes in [deg].
%       lon - row vector of doubles specifying longitudes in [deg].
%
%   Other m-files required: getHalversineDistance
%   Subfunctions: GPXRead_Custom
%   MAT-files required: none
%
%   See also: gpxread (requires Mapping Toolbox), GPXRead_Custom,
%   getHalversineDistance, fscanf

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 25-Oct-2020; Last revision: 06-Nov-2020
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved.

%% Read GPX data:
%Ensure that filename includes file extension:
if ~strcmpi(fileName(end-3:end),'.gpx')
    fileName = [fileName '.gpx'];
end

if exist(fileName, 'file') == 2
    if mapTBpresent %If the mapping toolbox is installed, use 'gpxread'
        Data = gpxread(fileName,'FeatureType','track','index',1);
        lat = Data.Latitude;
        lon = Data.Longitude;
    else %If this toolbox is not installed, use GPXRead_custom (less robust)
        [lat, lon] = GPXRead_custom(fileName);
    end
else
    error(['The specified file ' fileName ' cannot be found' newline...
        'Please ensure the filename is entered correctly.']);
end

%% Check whether GPX file is OSM based

%Read data from GPX file as plain text:
fid = fopen(fileName);
GPXdata = fscanf(fid, '%s');
fclose(fid);

%Search for OpenStreetMap references
OSMbool = contains(GPXdata,{'openstreetmap','project-osrm'},'IgnoreCase',true);

%If no OpenStreetMap references where found, throw an error:
if ~OSMbool
    error(['The input file ' fileName ' was probably not generated using an OpenStreetMap-based service. ' newline...
        'Please ONLY use OSM-based .gpx-file generators. Preferably, new routes can be downloaded via <a href="http://map.project-osrm.org/">OSRM</a>.']);
end

%% Distance check:
%Calculate route distance:
Distance = getHalversineDistance(lat',lon');

% Check whether the route distance exceeds maxDistance:
if Distance(end) > maxDistance
    warning(['The total length of the input route exceeds ' num2str(maxDistance/1e3) ' [km]. Parsing of the' newline...
             'OpenStreetMap data for this route might take a very long time or might fail because of' newline...
             'Out-Of-Memory errors. Possibly try dividing this route into multiple smaller routes.']);
end

end

function [lat, lon] = GPXRead_custom(fileName)
%GPXREAD_CUSTOM - Reads the GPS-coordinates from the .gpx-file specified by
% 'fileName'
%
%   Syntax:
%       [lat, lon] = GPXREAD_CUSTOM(fileName)
%
%   Description:
%       GPXREAD_CUSTOM() - Loard a GPS-track from a .gpx-file. Note that
%       this function works by 'manually' searching for certain
%       text-features in the .gpx-file that mark the occurance of
%       gps-coordinates. The .xml-format of the file is not respected.
%       Therefore, this function is not expected to be very robust with
%       resptect to different types of .gpx-files and should only be used
%       in case 'gpxread' (from the MATLAB mapping toolbox) is not
%       available.
%    
%   Inputs:
%       fileName - String describing the path and filename of a .gpx-file.
%
%   Outputs:
%       lat - column of doubles indicating the latitude coordinates [deg]
%       lon - column of doubles indicating the longitude coordinates [deg]
%
%   Other m-files required: none
%   Subfunctions: none
%   MAT-files required: none
%
%   See also: textscan, find, gpxread (= alternative)

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 05-Nov-2020; Last revision: 05-Nov-2020
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved.

%% Open the .GPX-file 
%using the xml-end tag '>' as delimiter.
fid = fopen(fileName);
    GPXFileData = textscan(fid,'%s','Delimiter','>');
fclose(fid);

%% Read GPS coordinates from file.

%Determine the indices of lines that contain track points:
trkptLines = strncmp(GPXFileData{:},'<trkpt',6);

%Store all these lines in a cell array:
relevantLines = {GPXFileData{:}{trkptLines}}';

%Of each of the lines that contain a track point, read the two numbers:
relevantData = textscan([relevantLines{:}],'%s lat="%f" lon="%f"');

%Of this data, the second colum is the latitude and the 3rd the longitude:
lat = relevantData{2}';
lon = relevantData{3}';

end