function trk = velocityProfilePredicion( route, driver, vehicle, OSMfolder, plotbool )
%velocityProfilePredicion - Gives an estimated velocity profile from
%coordinates of a specified route
%
%   Syntax:
%      [trk2] = velocityProfilePredicion( route, driver, OSMfolder)
%
%   Description:
%       velocityProfilePredicion() - Returns a velocity profile along
%       a certain route.
%       The function uses OpenStreetMap to find the kind of road, the
%       maximum allowed speed, the presence of crossings and traffic signs.
%       Combining this information with the driver information. 
%    
%   Inputs:
%       route -  struct with the fields 'lat' and 'lon' that specify the
%                route which is to be anlysed. It is advisable to plan the
%                routes through http://map.project-osrm.org/, as this route
%                planner also uses OpenStreetMap. 
%       driver - stuct containing the fields 'ay','acc','dec', and
%                'speed_bias'. These fields characterize the driver behaviour.
%                This behaviour influences the generated velocity profile
%                in various ways. For details, a reference to [2] is made.
%       vehicle -struct containing the field 'maxVelocity' that indicates
%                the maximum vehicle velocity in [km/h]. 
%       OSMfolder - Path of the folder where the OSM-maps can be stored and
%                found
%       plotbool - boolean indicating whether a plot will be generated or
%                not
%
%   Outputs:
%       trk   - Struct containing the fields:
%           time: [s] Equi-distant time vector
%            dis: [m] Corresponding displacement along route
%            vel: [m/s] Corresponding estimated vehicle velocity
%            lat: [deg] Latitude coordinates
%            lon: [deg] Longitude coordinates
%
%   Subfunctions: none 
%   MAT-files required: downloadOSMmap, processOSMmap (requires [2]),
%   getCornerCurvature 
%   Other files required: OpenStreetMapFunctions, as described in [2],
%   should be part of the current working directory.
%
%   See also: downloadOSMmap, processOSMmap, getCornerCurvature 
%
%   References:
%   [1] C.J.J. Beckers, M. Paroha, I.J.M. Besselink, and H. Nijmeijer, "A
%   Microscopic Energy Consumption Prediction Tool for Fully Electric
%   Delivery Vans," in EVS33 International Battery, Hybrid and Fuel Cell
%   Electric Vehicle Symposium, 2020.
%   [2]  Ioannis Filippidis. Openstreetmap functions
%   (https://github.com/johnyf/openstreetmap) Github, 2016. 
%   [Retrieved August 31, 2020].

%   Author: Tim Geraedts, Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Date: 9-June-2019; Last revision: 24-Sep-2020
%
%   Copyright (c) 2020, T.A.G.H.Geraedts
%   All rights reserved.

%% Set parameters
dis_step = 0.5;  %[m] %Discretization step in distance domain. Should be relatively small
fs = 10;         %[Hz]%Sampling frequency for time-integrations. Should be realtively high

Dt = (1/5)*dis_step/(130/3.6); %[s] timestep for time-integration 
                             %(criterium: at least 5 integration points
                             %within the specified distance-step if the
                             %vehicle is travelling at 130 km/h.

disp('===================== Velocity Profile Prediction has started.. ======================');                            
                          
%% Get Relevant Map Data:
%Make sure the required map-files are in the folder specified by OSMfolder
%If these are not present already, they will be downloaded
openstreetmap_filename = downloadOSMmap(route,OSMfolder);

%Proces the available map data, and store the RELEVANT details in a struct:
map = processOSMmap(route,openstreetmap_filename,plotbool);

disp('Predicting velocity profile...');

%% Determine velocity profile in distance domain
%Based on the information contained in 'map', a velocity profile will be
%estimated. This is done based ondifferent criteria:
% * Local corner radius
% * Maximumum speed limit

%Maximum velocity due to corner radius:
map.cRad = 1./abs(getCornerCurvature(map.lat,map.lon,false));
map.cSpeed = sqrt(driver.ay*map.cRad);

%Maximum velocity due to speed limit (with several assumptions)
map.speedlim = zeros(size(map.dis));
for i = 1:numel(map.speedlim)-1
    if isnan(map.wayInfo(i).maxspeed) %if no speed limit is known
        if strcmp(map.wayInfo(i).highway,'motorway') %then if the wayInfo indicates 'motorway'
            map.speedlim(i) = vehicle.maxVelocity/3.6; %assume maximum vehicle speed
        elseif i > 1                  %if this is not the first way of the route
            map.speedlim(i) = map.speedlim(i-1); %then assume speed limit of previous way
        else                          %if this is the first way of the route, and it is not 'motorway'
            map.speedlim(i) = 30/3.6; %then assume urban speedlimit (30 km/h)
        end
    else
        map.speedlim(i) = map.wayInfo(i).maxspeed/3.6; %assign speed limit as read from map data
    end
end

%CUSTOM ADDITION: Make Max speed at TU/e campus 30 km/h:
map.speedlim(map.lon > 5.48541 & map.lon < 5.49774 & map.lat > 51.44650 &  map.lat < 51.4501 & map.speedlim>30/3.6) = 30/3.6;

%Include a possible driver speed bias in the maximum speed:
map.speedlim = (100 + driver.speed_bias)/100 * map.speedlim;

%AFTER the driver limit has been taken into account, also
%Limit the speedlimit based on the prescribed maximum vehicle velocity;
map.speedlim = min(map.speedlim,vehicle.maxVelocity);

%% Convert distance-based velocity profile to a time based profile
%Define a dense distance grid:
disgrid = [0:dis_step:map.dis(end)]';

%Interpolate max speed limit to this distance grid:
vmax = interp1(map.dis,map.speedlim,disgrid,'previous');

%Make velocity zero at locations of traffic signals:
[~,traffsignindex] = min(abs(disgrid-map.dis(map.traffsign)'));
vmax(traffsignindex) = 0;

%Reduce velodity near corners:
vmax = min(vmax,interp1(map.dis,map.cSpeed,disgrid,'linear'));

%% Account for the limited vehicle acceleration
%Secondly, the vehicle acceleration is limited. To this end, numerical time
%integration is peformed between each set of consecutive set of point in
%the distance grid.

vel = zeros(size(disgrid)); %initialize velocity vector
vel(1) = 0; %assume to start at standstill

%forward loop over all data points:
for i = 2:length(disgrid)

    %Non-constant acceleration (numerical time-integration)
    t = 0;
    s = 0;
    v_int = vel(i-1);
    while v_int <= vmax(i) && s <= diff(disgrid(i-1:i))
        t     = t     +              Dt;
        s     = s     +   v_int     *Dt + 0.5*driver.acc(v_int)*Dt^2;
        v_int = v_int + driver.acc(v_int)*Dt;
    end
    
    vel(i) = min(v_int,vmax(i));
    
end

%backward loop over all data points:
vel(end) = 0; %assume trip ends at standstill
for i = length(disgrid)-1:-1:1
    
    %Non-constant acceleration (numerical time-integration)
    t = 0;
    s = 0;
    v_int = vel(i+1);
    while v_int <= vmax(i) && s >= -diff(disgrid(i:i+1))
        t     = t     -              Dt;
        s     = s     -  v_int      *Dt + 0.5*driver.dec(v_int)*Dt^2;
        v_int = v_int                   +     driver.dec(v_int)*Dt;
    end
    
    vel(i) = min([v_int,vel(i),vmax(i)]);
    
end

%interpolate lat lon to dense distance grid to correspond to the determined
%velocity profile:
lat = interp1(map.dis,map.lat,disgrid);
lon = interp1(map.dis,map.lon,disgrid);

%% Interpolate data to time-grid
% Define the time grid corresponding to the dense distance grid:
timegrid = [0; cumsum( 2*diff(disgrid) ./ (vel(1:end-1)+vel(2:end)) )];

%% Store data
%Define the final required (equidistant) time grid:
trk.time = [0:1/fs:timegrid(end)]';

%Interpolate all data to this required time grid:
trk.dis  = interp1(timegrid,disgrid,trk.time);
trk.vel  = interp1(timegrid,  vel  ,trk.time);
trk.lat  = interp1(timegrid, lat   ,trk.time);
trk.lon  = interp1(timegrid, lon   ,trk.time);

disp('=============== Velocity Profile Prediction has completed succesfully. ===============');

end %function
