function Pptloss = Ppt_loss_eVan( Wwheel, Twheel )
%PPT_LOSS_EVAN - Gives the powertrain losses of the Voltia eVan
%in [W] as function of wheel angular velocity and wheel torque.
%
%   Syntax:
%       Pptloss = PPT_LOSS_EVAN(Wwheel, Twheel)
%
%   Description:
%       PPT_LOSS_EVAN() - Returns the the powertrain losses in [W] as
%       function of angular wheel velocity and wheel torque. The losses are
%       calculated using separate expressions for the driving part (Twheel
%       > 0) and the regenerative part (Twheel < 0). The expressions are
%       emperical and originate from measurements.
%    
%   Inputs:
%       Wwheel - wheel angular velocity [rad/s]
%       Twheel - wheel torque [Nm]
%
%   Outputs:
%       Pptloss - Powertrain losses [W]
%
%   Other m-files required: none
%   Subfunctions: none
%   MAT-files required: none
%
%   See also: 

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 24-Mar-2019; Last revision: 24-Apr-2019 (14:30)
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved.

%% Set parameters
% Rename variables:
x = Wwheel;
y = Twheel;

%Define bounaries

ymax    = 2500; %[Nm] max torque at wheel
xmax    = 110;  %[rad/s] max wheel speed
xminreg = 4.3;  %[rad/s] min wheel speed for regen
Pmax    = 95e3; %[W] max power at wheel

%% Define coefficients of emperical functions
%Coefficients of Pidle, which is the  power that is lost when Twheel = 0.
%This power therefore only depends on the velocity x.
Pidle_coeff =[ 0.0051   -0.4540   42.7724 ];

%If torque is positive: (driving force)
     %Coefficients (with 95% confidence bounds):
       p10 =       5.956;  %(4.902, 7.01)
       p11 =      0.1653;  %(0.09268, 0.2379)
       p20 =  -0.0006802;  %(-0.001795, 0.0004347)
       p12 =  -5.819e-05;  %(-0.0001018, -1.456e-05)
       p21 =   -0.006035;  %(-0.007844, -0.004226)
       p22 =   1.633e-06;  %(5.308e-08, 3.214e-06)
       p30 =    6.66e-07;  %(3.268e-07, 1.005e-06)
       p31 =   4.271e-05;  %(3.073e-05, 5.469e-05)
       p32 =  -6.095e-09;  %(-1.532e-08, 3.135e-09)
       p33 =  -2.543e-12;  %(-9.29e-12, 4.205e-12)
       
Pptloss_drive =  p10.*y+p11.*x.*y+p20.*y.^2+p12.*x.*y.^2+p21.*x.^2.*y+p22.*x.^2.*y.^2+...
                    p30.*y.^3+p31.*x.^3.*y+p32.*x.^3.*y.^2+p33.*x.^3.*y.^3+...
                    Pidle_coeff(1)*x.^3+Pidle_coeff(2)*x.^2+Pidle_coeff(3)*x;
       
%If torque is negative (Regenerative braking)
     %Coefficients (with 95% confidence bounds):
       p10 =      -1.822;  %(-3.765, 0.1215)
       p11 =      -0.273;  %(-0.3955, -0.1505)
       p20 =    0.001091;  %(-0.0002059, 0.002388)
       p12 =  -2.153e-05;  %(-0.0001007, 5.767e-05)
       p21 =    0.004675;  %(0.002072, 0.007278)
       p22 =  -8.263e-07;  %(-3.138e-06, 1.485e-06)
       p30 =  -9.798e-08;  %(-4.042e-07, 2.082e-07)
       p31 =  -2.793e-05;  %(-4.436e-05, -1.151e-05)
       p32 =   7.491e-10;  %(-1.324e-08, 1.474e-08)
       p33 =  -7.934e-12;  %(-1.398e-11, -1.885e-12)
Pptloss_regen = p10.*y+p11.*x.*y+p20.*y.^2+p12.*x.*y.^2+p21.*x.^2.*y+p22.*x.^2.*y.^2+...
                    p30.*y.^3+p31.*x.^3.*y+p32.*x.^3.*y.^2+p33.*x.^3.*y.^3+...
                    Pidle_coeff(1)*x.^3+Pidle_coeff(2)*x.^2+Pidle_coeff(3)*x;
     
%% Combine driving and regen in one expression:                   
Pptloss = (y>=0).*Pptloss_drive + (y<0).*Pptloss_regen; %powertrain losses in [w]

% Limit the results
Pptloss(abs(y)    > ymax) = NaN; %eliminate values that exceed max torque
Pptloss(x         > xmax) = NaN; %eliminate values that exceed max speed
Pptloss(abs(x.*y) > Pmax) = NaN; %eliminate values that exceed max power
%Pptloss(x < xminreg & y<0) = NaN; %eliminate regen values at low velocities these where not measured, as vehicle does not regen here
end