function par = parameters_driver( FileName )
%PARAMETERS_DRIVER Loads parameters defining the driver behaviour.
%
%   Syntax:
%       par = PARAMETERS_DRIVER( FileName )
%
%   Description:
%       PARAMETERS_DRIVER() - returns a struct 'par' containing several
%       driver specific parameters. All parameters are in SI units and are
%       loosely based on the trips driven with the Volta eVan in Eindhoven
%       on 2018-10-09 and 2018-10-10.
%    
%   Inputs:
%       FileName - The filename of an .xls-file containing the specific
%       numeric values to the driver parameters.
%
%   Outputs:
%       par - struct containing several parameters. Some of these
%       parameters are stored as function handles. These are non-constant
%       parameters that depend on an input parameter (i.e. velocity).
%
%   Other m-files required: none
%   Subfunctions: none
%   MAT-files required: none
%
%   See also: struct, xlsread

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 18-Jul-2019; Last revision: 06-Aug-2019
%
%   Copyright (c) 2020, C.J.J.Beckers, T.A.G.H.Geraedts
%   All rights reserved.

%% Vehicle specifications

%Load data from Excel file
Excel = xlsread(FileName,'Sheet1','','basic');
Excel(:,1) = []; %Remove first column of Excel table (corresponds to the plot in the file)

% Assign variables
par.ay = Excel(2);             %[m/s^2] max lateral acceleration in corners

max_long_acc     = Excel(3);   %[m/s^2] max longitudinal acceleration (forward) 
max_long_acc_vel = Excel(6);   %[m/s] max velocity at which max long acc still occurs (forward)
par.acc = @(v) min(max_long_acc,max_long_acc*max_long_acc_vel./v); %[m/s^2] longitudinal acceleration as function of velocity (

max_long_dec     = Excel(4);   %[m/s^2] max longitudinal deceleration (backward)
max_long_dec_vel = Excel(7);   %[m/s] max velocity at which max long dec still occurs (backward)
par.dec = @(v) min(max_long_dec,max_long_dec*max_long_dec_vel./v); %[m/s^2] longitudinal deceleration as function of velocity (

par.speed_bias = Excel(9);     %[%]  Velocity bias of driver (=10% is faster driving than allowed -10% is slower driving than allowed)

% %% Plot acceleration/deceleration as function of velocity: (only for
% % manual checking)
% v = 1:130/3.6;
% figure; hold on;
% plot(v,par.acc(v));
% plot(v,-par.dec(v));
end
