function par = parameters_vehicle( FileName )
%PARAMETERS_VEHICLE - Loads vehicle parameters from an Excel file into a
%struct.
%
%   Syntax:
%       par = PARAMETERS_VEHICLE( FileName )
%
%   Description:
%       parS_VEHICLE() - returns a struct 'par' containing several vehicle
%       specific parameters. All parameters are in SI units and are
%       specific to the vehicle described in the excelfile specified by
%       FileName.
%    
%   Inputs:
%       FileName - The filename of an .xls-file containing the specific
%       numeric values to the vehicle parameters.
%
%   Outputs:
%       par - struct containing several parameters. Some of these
%       parameters are stored as function handles. These are non-constant
%       parameters that depend on an input parameter. 
%
%   Other m-files required: none
%   Subfunctions: none
%   MAT-files required: none
%
%   See also: struct, xlsread

%   Author: Camiel Beckers
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 18-Jul-2019; Last revision: 23-Sep-2019
%
%   Copyright (c) 2020, C.J.J.Beckers, T.A.G.H.Geraedts
%   All rights reserved.

%% Vehicle specifications

%Load data from Excel file
Excel = xlsread(FileName,'Sheet1','','basic');

%Assign data from Excel to vehicle parameters:
par.mVehicle = Excel(2,2);     % [kg] Total mass of the vehicle
par.mCargo   = Excel(3,2);     % [kg] Total mass of cargo (incl. persons) 

par.mwheel   = Excel(5,2);     % [kg] Mass of one wheel
par.rTyre    = Excel(6,2);     % [m]  Tyre radius

par.gearRatio= Excel(8,2);     % [-]  Gear ratio between motor and wheel
par.Jrotor   = Excel(9,2);     % [kgm^2] Motor rotor inertial
par.Jwheel   = 0.5*par.mwheel*par.rTyre^2;  % [kgm^2] Wheel inertia

par.mEff     = par.mVehicle + 4*par.Jwheel/par.rTyre^2 + par.Jrotor*par.gearRatio^2/par.rTyre^2;
                               % [kg] Effective moving mass of vehicle

par.Af       = Excel(11,2);    % [m2] Frontal area        
par.Cd       = Excel(12,2);    % [-] Aerodynamic dragcoefficient (average from measurements)
par.DeltaCd  = @(beta) min( Excel(14,2)/Excel(14,3)*abs(beta), Excel(14,2) );

%Rolling resistance properties
par.fr_vel1      = Excel(25,2);% [km/h] velocity limit between bad and medium asphalt
par.fr_vel2      = Excel(27,2);% [km/h] velocity limit between medium and good asphalt
par.fr_high      = Excel(24,3);% [-] rolling resistance coefficient (bad asphalt)
par.fr_mid       = Excel(26,3);% [-] rolling resistance coefficient (medium asphalt)
par.fr_low       = Excel(28,3);% [-] rolling resistance coefficient (good asphalt)

par.P_aux       = Excel(16,2); % [W] Auxiliary power (assumed constant)
par.maxVelocity = Excel(20,2); %[km/h] maximum vehicle velocity

par.maxRegenWhlForce = Excel(18,2); % [N] max regen force at wheel (as measured from dyno)
par.Fwhl_regenLimit = @(v_ms) max( par.maxRegenWhlForce, v_ms.*(par.maxRegenWhlForce)/(15/3.6)+1); 
%[N] Force limit that can maximally be applied by regenerative braking.
%Defined as function of vehicle speed [m/s]

end %function