%TUe_MECPRO - Runs the Microscopic Energy Consumption PRediction tOol
%according to the methodology described in [1].
%
%   Description:
%       TUe_MECPRO is generally devided in three sections:
%           1. Contains the relevant settings for the energy consumption
%           prediction. These can be edited by the user. Most imporatantly,
%           the vehicle environment and driver parameters are loaded into
%           the script here.
%           2. Calls all the functions required for performing the
%           calculations related to the energy consumption prediction. For
%           the details regarding these calculations, a reference is made
%           to [1].
%           3. Plots and stores the results in a newly created folder
%
%   Output:
%       TUe_MECPRO generates several outputs. First of all, the progress of
%       the calculations is written to the command window. Secondly,
%       several figures are produced and stored in './Figures'. Lastly, the
%       solution of the energy consumption prediction is written to the
%       struct named 'sol'.
%
%   [1] C.J.J. Beckers, M. Paroha, I.J.M. Besselink, and H. Nijmeijer, "A
%   Microscopic Energy Consumption Prediction Tool for Fully Electric
%   Delivery Vans," in EVS33 International Battery, Hybrid and Fuel Cell
%   Electric Vehicle Symposium, 2020.
%
%   Other m-files required: parameters_vehicle, parameters_environment,
%   parameters_driver, velocityProfilePredicion, filterVelocity,
%   getSRTMelevation, getRelativeWind, energyConsumptionPrediction
%   MAT-files required: none
%
%   See also: parameters_vehicle, parameters_environment,
%   parameters_driver, velocityProfilePredicion, filterVelocity,
%   getSRTMelevation, getRelativeWind, energyConsumptionPrediction

%   Authors: Camiel Beckers, Tim Geraedts, Igo Besselink, Henk Nijmeijer
%   email: c.j.j.beckers@tue.nl
%   Website: https://research.tue.nl/en/persons/camiel-jj-beckers
%   Date: 18-Jul-2019; Last revision: 22-Dec-2020
%
%   Copyright (c) 2020, C.J.J.Beckers
%   All rights reserved.

%Clear workspace
clc; clearvars; close all;

%% 1. Define paths and settings (to be edited by user)
folders.current = pwd;      %set current path
addpath('./Functions');     %add folder containding functions

%Add OpenStreetmap Funcions by Ioannis Filippidis:
addpath('./Functions/OpenStreetMap_Functions'); 
%Add Readhgt by Francois Beauducel:
addpath('./Functions/Readhgt'); 
%Add xml2struct by Wouter Falkena:
addpath('./Functions/xml2struct');
%Add FilterM by Jan Simon (replaces signalprocessing toolbox)
addpath('./Functions/FilterM');

addpath(genpath('./Data'));          %Folder containing various types of map data
addpath(genpath('./Input_files'));   %Folder containing user-defined input files

folders.SRTMmaps = './Data/SRTM/';     %Folder containing SRTM maps
folders.OSMmaps  = './Data/OSM/';      %Folder containing OSM maps
folders.Routes   = './Input_files/routes'; %Folder containing the route that is to be analysed


% ======================= Change Input Route Here ======================= %
route.RouteName ='route';      %Name of the route that will be analysed
% ======================================================================= %


% ================== Change Algorithm Settings Here ===================== %
%Settings for the Energy Consumption Prediction Script
settings.includeSlope   = true; %Include variable road slope
settings.includeWeather = false; %Include wind effects & weather variablility
settings.variableFr     = true; %Include variable rolling resistance (depending on vehicle velocity)
settings.dateToday      = datevec(datetime(now,'ConvertFrom','datenum')); %Set current date

%Visualisation Settings:
settings.makeIntermediatePlots = true; %Generate intermediate plots
% ======================================================================= %

%% Print logo and perform MATLAB version check
disp(['      _____ _   _   __       __  __ _____ ____ ____  ____   ___       ___   _' newline  '     |_   _| | | | / /__    |  \/  | ____/ ___|  _ \|  _ \ / _ \     / _ \ / |' newline '       | | | | | |/ / _ \   | |\/| |  _|| |   | |_) | |_) | | | |   | | | || |' newline '       | | | |_| / /  __/   | |  | | |__| |___|  __/|  _ <| |_| |   | |_| || |' newline '       |_|  \___/_/ \___|   |_|  |_|_____\____|_|   |_| \_\\___/     \___(_)_|' newline])
       
% Toolbox check
settings.mapToolboxPresent = checkToolboxes({'Mapping Toolbox'});

% Internet check: try to connect to a specified website
checkInternet('mathworks.com');

%% Load parameters
vehicle     = parameters_vehicle(    'parameters_vehicle.xls'    ); %load vehicle parameters
environment = parameters_environment('parameters_environment.xls'); %load environment parameters
driver      = parameters_driver(     'parameters_driver.xls'     ); %load driver parameters

%% Define Track (which contains all info about the route that is travelled)

%Read route from GPX file. Route only consists of lat/lon coorindates:
[route.lat,route.lon] = readGPXfile(fullfile(folders.Routes,route.RouteName),...
    settings.mapToolboxPresent,100e3);

%Display route info to user:
disp(['Route ' fullfile(folders.Routes,route.RouteName) ' will be analysed.']);
disp('New routes can be downloaded via <a href="http://map.project-osrm.org/">OSRM</a>. Please ONLY use OSM-based .gpx-file generators.');

%Perform Velocity Profile Prediction:
trk = velocityProfilePredicion(route,driver,vehicle,folders.OSMmaps,settings.makeIntermediatePlots);
if settings.makeIntermediatePlots
    saveas(gcf,['./Figures/' route.RouteName '_Route_data.eps'],'epsc')
    saveas(gcf,['./Figures/' route.RouteName '_Route_data.fig'],'fig')
end
        
trk.date = settings.dateToday; %Assign current date to track

%% Determine Additional Route Information
%filter track velocity (vel) and determine longitudinal acceleration (acc)
[trk.dis,trk.vel,trk.acc] = filterVelocity(trk);

%determine elevation (elev) and gradient (grad) from route
if any(trk.lat) && any(trk.lon) && settings.includeSlope
    [trk.elev, trk.elevraw, trk.grad] = getSRTMelevation(trk, folders.SRTMmaps, settings.makeIntermediatePlots);
    if settings.makeIntermediatePlots
        saveas(gcf,['./Figures/' route.RouteName '_elevation_map.eps'],'epsc')
        saveas(gcf,['./Figures/' route.RouteName '_elevation_map.fig'],'fig')
    end
    disp('Road slope has been determined from STRM elevation map.');
else
    disp('Road slope is assumed zero');
end

%determine relative wind direction and up-to-date air density.
if any(trk.lat) && any(trk.lon) && settings.includeWeather
    try
        [trk.relWindMag,trk.relWindBeta,environment.rho] = getRelativeWind(trk,environment.windfactor,settings.makeIntermediatePlots);
        if settings.makeIntermediatePlots
            saveas(gcf,['./Figures/' route.RouteName '_wind_direction.eps'],'epsc')
            saveas(gcf,['./Figures/' route.RouteName '_wind_direction.fig'],'fig')
        end
        disp('Relative wind direction and air density were obtained from OpenWeatherMap.');
    catch ME
        warning(['Weather information could not be determined.' newline 'Probably, OpenWeatherMaps cannot be reached or the API-key supplied in OpenWeatherMap_key.txt is invalid.' newline ME.message]);
        disp('Weather data is not taken into account. No wind and default air density is assumed.');
        clear ME
    end
else
    disp('Weather data is not taken into account. No wind and default air density is assumed.');
end

%% Perform Energy Consumption Prediction
sol = energyConsumptionPrediction(trk,vehicle,environment,settings);

%% Plot Results

%Estimated slope
if settings.includeSlope && settings.makeIntermediatePlots
    figure;
    ax1 = subplot(2,1,1); hold on;
    plot(trk.dis/1e3,trk.elevraw,'k.');
    plot(trk.dis/1e3,trk.elev);
    xlabel('Travelled distance [km]');
    ylabel('Elevation [m]');
    legend('raw data','filtered data');
    ax2 = subplot(2,1,2); hold on;
    plot(trk.dis(1:end-1)/1e3,tan(trk.grad)*100);
    xlabel('Travelled distance [km]');
    ylabel('Gradient [%]');
    linkaxes([ax1,ax2],'x');
    xlim([trk.dis(1)/1e3 trk.dis(end)/1e3])
    clearvars ax1 ax2
    saveas(gcf,['./Figures/' route.RouteName '_slope.eps'],'epsc')
    saveas(gcf,['./Figures/' route.RouteName '_slope.fig'],'fig')
end

%Assumed and actual driver acceleration/deceleration
if settings.makeIntermediatePlots
    figure; hold on;
    plot(trk.vel(1:end-1)*3.6,trk.acc,'.');
    xlim([0 130]);
    ylim([-2.5 2.5])
    plot([0:130],driver.acc([0:130]/3.6));
    plot([0:130],-driver.dec([0:130]/3.6));
    legend('simulated velocity profile','acceleration model','deceleration model');
    xlabel('Vehicle velocity [km/h]');
    ylabel('Vehicle acceleration [m/s^2]');
    saveas(gcf,['./Figures/' route.RouteName '_acceleration.eps'],'epsc')
    saveas(gcf,['./Figures/' route.RouteName '_acceleration.fig'],'fig')
end

%Predicted Velocity Profile
figure; hold on;
plot(trk.dis/1e3,trk.vel*3.6);
xlim([trk.dis(1)/1e3 trk.dis(end)/1e3])
xlabel('Travelled distance [km]');
ylabel('Predicted velocity [km/h]');
ylim([0 130])
saveas(gcf,['./Figures/' route.RouteName '_velocity.eps'],'epsc')
saveas(gcf,['./Figures/' route.RouteName '_velocity.fig'],'fig')

%Predicted Energy Consumption
figure; hold on;
plot( sol.dis/1e3,sol.E_discharge/3.6e6);
plot( sol.dis/1e3,sol.E_regen/3.6e6);
xlim([0 trk.dis(end)/1e3]);
xlabel('Travelled distance [km]');
ylabel('Energy [kWh]');
legend('Discharged','Regenerated','location','NW');
saveas(gcf,['./Figures/' route.RouteName '_energy.eps'],'epsc')
saveas(gcf,['./Figures/' route.RouteName '_energy.fig'],'fig')